--------------------------------------------------------------------------------
-- Company: Trenz Electronic GmbH
-- Engineer: Antti Lukats, Oleksandr Kiyenko, John Hartfiel
--
-- TEBF808 Master CPLD U17, SC.Sch, first in the JTAG Chain
--
--
-- Revision     : REV07
-- Release Date : 2019-05-03
-- Supported PCB Revision: 03,04
-- changes REV06 to REV07:
--  - complete rework
--  - add inter CPLD RGPIO
--  - new LED debugging sequencing
--  - simple UART output with Revision Number and some statistic on second XMOD
-- changes REV05 to REV06:
--  - BUGFIX: renamed SC19 to SC17 -> loc was set SC17 in REV05
--  - connect FMC and XMOD JTAG
--  - add CAN to PL
--  - RGPIO Pin changes
-- changes REV04 to REV05:
--  - SD WP forwarded second CPLD
--  - PL access to XMOD LED
--------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.STD_LOGIC_arith.all;
use ieee.STD_LOGIC_unsigned.all;
use ieee.STD_LOGIC_misc.all;
use IEEE.NUMERIC_STD.ALL;
library machxo2;
use machxo2.all;
--------------------------------------------------------------------------------
entity top is
   generic (
     CPLD_REV : string := "TEBF0808-x_CPLD-REV0?DEV"
   );
port (
  --Power
	DP_EN				: out STD_LOGIC; -- Enable power to DP out LDO  
  --Enables
	OSC_EN				: out STD_LOGIC;
  --Resets
	I2C_RST				: out STD_LOGIC;
	USB_RST				: out STD_LOGIC;
	--USB_TRST	   		: out STD_LOGIC; -- do not care
	USBH_RST			    : out STD_LOGIC;      --(PCB_REV02, NC--> other USB HUB), REV02:comment  
  ---
	-- USB_TRST			  : out STD_LOGIC;      --(PCB_REV02, REV03,REV04 -> PCB_REV04_DET and input), REV02:uncomment and loc constrain
	-- PCB_REV04_DET		: in STD_LOGIC;      --(REV02->USB_TRST and output, PCB_REV03->float, REV04->GND ), REV02:comment 
  
  --XMOD JTAG
	C_TMS		 		: in  STD_LOGIC;
	C_TCK		 		: in  STD_LOGIC;
	C_TDI		 		: in  STD_LOGIC;
	C_TDO		 		: out STD_LOGIC;

	--	
	FMC_VID0	   		: out STD_LOGIC;
	FMC_VID1	   		: out STD_LOGIC;
	FMC_VID2	   		: out STD_LOGIC;
	FMC_CLKDIR	 		: in  STD_LOGIC;
	
	FMC_TCK	   			: out STD_LOGIC;
	FMC_TMS	   			: out STD_LOGIC;
	FMC_TDI	   			: out STD_LOGIC;
	FMC_TDO	   			: in  STD_LOGIC;
		
	SDA		 			: in  STD_LOGIC;
	SCL		 			: in  STD_LOGIC;				
	SD_WP		 		: in  STD_LOGIC;				
				
	-- 20 pin JTAG
	JTAG_TRST			: in  STD_LOGIC;				
	JTAG_SRST			: in  STD_LOGIC;				 -- soft reset
	JTAG_TCK			: in  STD_LOGIC;				
	JTAG_RTCK			: out STD_LOGIC;				
	JTAG_TMS			: in  STD_LOGIC;				
	JTAG_TDI			: in  STD_LOGIC;				
	JTAG_TDO			: out STD_LOGIC;				
		
	CAN_TX		   		: out STD_LOGIC;
	CAN_RX		   		: in  STD_LOGIC;
	CAN_S		   		  : out STD_LOGIC;
	CAN_FAULT   		: in  STD_LOGIC;

	MIO26		   		: out  STD_LOGIC; --tck
	MIO27		   		: out  STD_LOGIC; --tdi
	MIO28		   		: in  STD_LOGIC;  --tdo
	MIO29		   		: out  STD_LOGIC; --tms
		
	SFP_LED0	   		: out STD_LOGIC;
	SFP_LED1	   		: out STD_LOGIC;
	SFP_LED2	   		: out STD_LOGIC;
	SFP_LED3	   		: out STD_LOGIC;
	--
	-- ETH PHY
	--	
	PHY_LED0	   		: in  STD_LOGIC;
	PHY_LED1	   		: in  STD_LOGIC;
	PHY_LED2	   		: in  STD_LOGIC;
	PHY_CLK	   			: in  STD_LOGIC;
	ETH_RST				: out STD_LOGIC;
	PHY_CONFIG	 		: out STD_LOGIC;
	
	JLED1		   		: out STD_LOGIC;
	JLED2A		   		: out STD_LOGIC;
	JLED2B		   		: out STD_LOGIC;
	XMOD_G				: in  STD_LOGIC; --  There is extra Button on XMOD      XMOD without green dot
	XMOD_E				: out STD_LOGIC; --  There is extra LED on XMOD E pin   XMOD without green dot
	
	XMOD_TXD			: out STD_LOGIC;
	XMOD_RXD			: in  STD_LOGIC;
	-- dummy		  		: out STD_LOGIC;

	-- to From DP
	DP_HPD				: in  STD_LOGIC;
	DP_DE				: out STD_LOGIC;
	DP_TX				: out STD_LOGIC;
	DP_RX				: in  STD_LOGIC;
	-- USB HUB	
	USBH_MODE0	   		: out STD_LOGIC;  --(PCB_REV02, NC--> other USB HUB), REV02:comment 
	USBH_MODE1	   		: out STD_LOGIC;  --(PCB_REV02, NC--> other USB HUB), REV02:comment 
-----------------------------------------------------
-- PL Side
--		PL_DP_HPD	  		: out STD_LOGIC;  -- IO 14
--		PL_DP_DE	   		: in  STD_LOGIC;   -- IO 15
--		PL_DP_TX	   		: in  STD_LOGIC;   -- IO 16
--		PL_DP_RX	   		: out STD_LOGIC;  -- IO 17

	SC10				: inout STD_LOGIC;  -- DP AUX
	SC11				: in  STD_LOGIC;  -- DP EN
	SC12				: out STD_LOGIC;  -- -- HPD shared?
	SC13				: out STD_LOGIC;  -- TX data
	SC14				: in  STD_LOGIC;  -- RX DATA
	SC15				: in  STD_LOGIC;  -- RX CLK
	SC16				: in  STD_LOGIC;  -- CAN_S

	SC17				: in  STD_LOGIC;  -- forwarded to XMOD LED
	SC18				: in  STD_LOGIC;  -- CAN_TX
	SC19				: out  STD_LOGIC;  -- CAN_RX

	CLK0				: in  STD_LOGIC;  -- --(PCB_REV02, differential P), REV02:comment 
	CLK1				: in  STD_LOGIC;  -- --(PCB_REV02, differential N), REV02:comment 

	-- inter SCx
	X0	  				: out  STD_LOGIC;  -- MTS dummy                     --1.8V 
	X1	  				: in  STD_LOGIC;  --  inter cpld RGPIO CLK                    --1.8V
	X2	  				: out  STD_LOGIC;  -- inter cpld RGPIO TX            --1.8V
	X3	  				: in  STD_LOGIC;  -- inter cpld RGPIO RX             --1.8V
	X4	  				: out STD_LOGIC;  -- SD WP to slave cpld             --1.8V
	X5	  				: in STD_LOGIC;  --   STM dummy           --1.8V
	X6	  				: out STD_LOGIC;  --  unused   --1.8V
	X7	  				: out STD_LOGIC;  --  unused   --1.8V
	X8	  				: in STD_LOGIC;  --       --3V3SB
	SW3		 			: in  STD_LOGIC;   -- 
	SW4		 			: in  STD_LOGIC;   -- 
	LED0				: out STD_LOGIC;
	LED1				: out STD_LOGIC;
	-- SFP Controls
	SFP1_TX_DIS			: out STD_LOGIC; --(PCB_REV02, NC), REV02:comment 
	SFP2_TX_DIS			: out STD_LOGIC --(PCB_REV02, NC), REV02:comment 
 );
end entity top;
--------------------------------------------------------------------------------
architecture translated of top is
--------------------------------------------------------------------------------

-------------------------------
component uart_id
  generic (
    DIVIDER : integer := 2518;  --9600:2518@24,18MHz CLK, --4800:5038@24,18MHz CLK,--2400:10075@24,18MHz CLK,--1200:20150@24,18MHz CLK,
    TXT_STRING : string := "TEBF0808-x_CPLD-REV07DEV";
    STAT_SIZE : integer :=6
  );
  port ( 
    CLK_IN : in std_logic;
    RESET_IN : in std_logic; -- one: RX-TX feedback, zero: start stream
    UART_TX_OUT : out std_logic;
    UART_RX_IN : in std_logic;
    --
    STAT_POR_IN    : in std_logic;
    STAT_POR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_POR_IN    : in std_logic;
    STAT_PS_POR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_SRST_IN    : in std_logic;
    STAT_PS_SRST_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PCIE_IN    : in std_logic;
    STAT_PCIE_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_USB_IN    : in std_logic;
    STAT_USB_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_ERR_IN    : in std_logic;
    STAT_PS_ERR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_ERRSTAT_IN    : in std_logic;
    STAT_PS_ERRSTAT_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_INIT_IN    : in std_logic;
    STAT_PS_INIT_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_MPSTM_IN   : in std_logic_vector(4-1 downto 0)
  );
end component;


component RGPIO_top is
  Generic (
    C_ADD_RESERVED  : integer range 0 to 1          := 0;--0 disabled 1 enabled
    C_TYP           : integer range 0 to 2          := 0  --0 Master, 1 Slave ,2 Both 
  );
  Port ( 
		-----------------------------------------------------
		--------------------FPGAIO-BUS Ports
		--
		-- Master 
		-- -external
		RGPIO_M_CLK   : out std_logic;   --  RGPIO Clock
		RGPIO_M_RX    : in  std_logic;   -- RGPIO RXD        
		RGPIO_M_TX    : out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_M_OUT   : out std_logic_vector(23 downto 0); --user data
    RGPIO_M_IN    : in std_logic_vector(23 downto 0); --user data
		-- -internal control
	  RGPIO_M_RESERVED_OUT            : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_RESERVED_IN             : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_SLAVE_ACTIVATION_CODE   : out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
	  RGPIO_M_ENABLE                  : in  std_logic; -- Enable RGPIO
	  RGPIO_M_USRCLK                  : in  std_logic; -- RGPIO CLK
	  RGPIO_M_RESET_N                 : in  std_logic; -- RGPIO Reset
		--  
		-- Slave 
		-- -external
		RGPIO_S_CLK   : in std_logic;  --  RGPIO Clock
		RGPIO_S_RX    : in  std_logic;   -- RGPIO RXD        
		RGPIO_S_TX    : out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_S_OUT   : out std_logic_vector(23 downto 0); --user data
    RGPIO_S_IN    : in std_logic_vector(23 downto 0); --user data
		-- -internal control
	  RGPIO_S_RESERVED_OUT  : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_RESERVED_IN   : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_MASTER_ACTIVATION_CODE   : out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
	  RGPIO_S_ENABLED   : out  std_logic -- Enable RGPIO
  
	);
end component RGPIO_top;

component OSCH
-- synthesis translate_off
generic (
	NOM_FREQ: string := "24.18"
);
-- synthesis translate_on
port ( 
	STDBY		: in  STD_LOGIC;
	OSC			: out STD_LOGIC;
	SEDSTDBY 	: out STD_LOGIC
);
end component OSCH;

attribute NOM_FREQ 		: string;
attribute NOM_FREQ of OSCinst0 : label is "24.18";

constant STAT_SIZE : integer := 6;



signal rgpio_in_mux_i : std_logic_vector(3 downto 0);
signal rgpio_in_data_i : std_logic_vector(23 downto 0);
signal rgpio_out_data_i : std_logic_vector(23 downto 0);



signal stat_trig_por_i : std_logic;
signal stat_trig_ps_srst_i : std_logic;
signal stat_trig_ps_por_i : std_logic;
signal stat_trig_pcie_i : std_logic;
signal stat_trig_usb_i : std_logic;
signal stat_trig_ps_init_i : std_logic;
signal stat_trig_ps_err_i : std_logic;
signal stat_trig_ps_errstat_i : std_logic;

signal stat_por_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_srst_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_por_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_pcie_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_usb_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_err_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_errstat_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_init_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_mpstm_i : std_logic_vector(3 downto 0);



signal osc_clk			: STD_LOGIC;
signal clk				: STD_LOGIC;
signal led_cnt				: STD_LOGIC_VECTOR(31 downto 0);
signal LED_Blink		: STD_LOGIC;
signal blink			: STD_LOGIC;
signal slow_blink	: STD_LOGIC;
signal blink_status		: STD_LOGIC_VECTOR(7 downto 0);
signal por_resetn		: STD_LOGIC; -- if this is LOW assert reset to all peripherals
signal rgpio_clk 		: STD_LOGIC; 		
signal rgpio_txd 		: STD_LOGIC; 		
signal rgpio_rxd 		: STD_LOGIC; 		
signal soc_rgpio_active 	: STD_LOGIC; 		

signal iclld_rgpio_clk 		: STD_LOGIC; 		
signal iclld_rgpio_txd 		: STD_LOGIC; 		
signal iclld_rgpio_rxd 		: STD_LOGIC; 		
signal iclld_rgpio_out 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal iclld_rgpio_in 		: STD_LOGIC_VECTOR(31 downto 0); 		
signal iclld_rgpio_active 	: STD_LOGIC; 
signal ps_srst_p_i    : std_logic_vector(33-1 downto 0) := (others => '0');
signal ps_por_p_i     : std_logic_vector(33-1 downto 0) := (others => '0');
signal ps_err_p_i     : std_logic_vector(33-1 downto 0) := (others => '0');
signal ps_err_stat_p_i: std_logic_vector(33-1 downto 0) := (others => '0');
signal ps_init_p_i    : std_logic_vector(33-1 downto 0) := (others => '0');
signal pcie_rst_p_i   : std_logic_vector(33-1 downto 0) := (others => '0');
signal usb_rst_p_i    : std_logic_vector(33-1 downto 0) := (others => '0');
signal por_p_i        : std_logic_vector(33-1 downto 0) := (others => '0');


signal usb_resetn		: STD_LOGIC;
signal intercpld_bpwr_i		: STD_LOGIC;

signal ps_srst_p      : std_logic;
signal ps_por_p       : std_logic;
signal ps_err_p       : std_logic;
signal ps_err_stat_p  : std_logic;
signal ps_init_p      : std_logic;
signal pcie_rst_p     : std_logic;
signal usb_rst_p      : std_logic;
signal por_p          : std_logic;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
BEGIN
--------------------------------------------------------------------------------
-- unused
-------------
X0 	<=  SCL or SDA  or PHY_CLK or CLK0 or CLK1 or X5;
X6 			<= '0';
X7			<= '1';

--------------------------------------------------------------------------------
--REV02 only:
-- USB_TRST	<= '1';     --(PCB_REV02, REV03,REV04 -> PCB_REV04_DET and input), REV02:add 
--------------------------------------------------------------------------------
-- SW4 control the FMC VADJ
-- - 010 = 1.8V
-- - 101 = 1.2V
-------------
FMC_VID0	<= '0' when SW4 ='0' else '1';
FMC_VID1	<= '1' when SW4 ='0' else '0';
FMC_VID2	<= '0' when SW4 ='0' else '1';
---- Sanity check to other CPLD
iclld_rgpio_in(1 downto 0) <= "10";
-- if 00 or 11 are sampled, other CPLD
-- must turn off FMC_VADJ regulator just in case
-- the VIDx is invalid

--------------------------------------------------------------------------------
-- Inter CPLD Resets and status signals
-------------
intercpld_bpwr_i <= X8; -- 1.8V inter clpd bank voltage disabled,
--did not use inter CPLD signals as long as 1.8V is not available on slave activation code is ok
ps_srst_p_i(0)     <='1'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(23); --pos ps soft reset
ps_por_p_i(0)      <='1'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(22); --pos ps power on reset
ps_err_p_i(0)      <='0'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(21); --pos Err flag
ps_err_stat_p_i(0) <='0'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(20); --pos Err stat flag
ps_init_p_i(0)     <='0'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0' or  por_p='1'   else iclld_rgpio_out(19); --pos Init flag
pcie_rst_p_i(0)    <='1'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(18); --pos pcie reset
usb_rst_p_i(0)     <='1'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(17); --pos usb reset 
por_p_i(0)         <='1'    when iclld_rgpio_active='0' or intercpld_bpwr_i='0'                 else iclld_rgpio_out(16) and (not iclld_rgpio_out(15)); --pos power on reset

delay_reg: process(clk)
begin
  if(rising_edge(clk)) then
    ps_srst_p_i(ps_srst_p_i'length-1 downto 1)<= ps_srst_p_i(ps_srst_p_i'length-2 downto 0);
    ps_por_p_i(ps_por_p_i'length-1 downto 1)<= ps_por_p_i(ps_por_p_i'length-2 downto 0);
    ps_err_p_i(ps_err_p_i'length-1 downto 1)<= ps_err_p_i(ps_err_p_i'length-2 downto 0);
    ps_err_stat_p_i(ps_err_stat_p_i'length-1 downto 1)<= ps_err_stat_p_i(ps_err_stat_p_i'length-2 downto 0);
    ps_init_p_i(ps_init_p_i'length-1 downto 1)<= ps_init_p_i(ps_init_p_i'length-2 downto 0);
    pcie_rst_p_i(pcie_rst_p_i'length-1 downto 1)<= pcie_rst_p_i(pcie_rst_p_i'length-2 downto 0);
    usb_rst_p_i(usb_rst_p_i'length-1 downto 1)<= usb_rst_p_i(usb_rst_p_i'length-2 downto 0);
    por_p_i(por_p_i'length-1 downto 1)<= por_p_i(por_p_i'length-2 downto 0);
  end if;
end process;

--resets must be high for min of *_p_i'length*1/CLK ~33*1/24,18MHz=1,365us (means 2 RGPIO transmissions)
ps_srst_p     <= and_reduce(ps_srst_p_i);
ps_por_p      <= and_reduce(ps_por_p_i);
ps_err_p      <= and_reduce(ps_err_p_i);
ps_err_stat_p <= and_reduce(ps_err_stat_p_i);
ps_init_p     <= and_reduce(ps_init_p_i);
pcie_rst_p    <= and_reduce(pcie_rst_p_i);
usb_rst_p     <= and_reduce(usb_rst_p_i);
por_p         <= and_reduce(por_p_i);

-- external resets:
por_resetn	<= (not por_p) and intercpld_bpwr_i; -- rests if reset comes from slave CPLD or 1.8V is missing
usb_resetn	  <= (not usb_rst_p) and por_resetn;

--------------------------------------------------------------------------------
-- USB
-------------
USBH_RST <= usb_resetn;
USB_RST <= usb_resetn;
USBH_MODE0	<= '1'; --, REV02:comment 
USBH_MODE1	<= '1'; --, REV02:comment 
--------------------------------------------------------------------------------
-- DP
-------------
DP_EN <= not por_resetn;
-- DP control from FPGA
SC10		    <= DP_RX when SC11 = '0' else 'Z';
DP_TX		    <= SC10; 
DP_DE 		  <= SC11; -- 1 = transmit
SC12		    <= DP_HPD;
--------------------------------------------------------------------------------
-- I2C
-------------
I2C_RST	 	  <= por_resetn;
--------------------------------------------------------------------------------
-- ETH
-------------
ETH_RST  	  <= por_resetn;
PHY_CONFIG 	<= '1';
--------------------------------------------------------------------------------
-- SD
X4 <= SD_WP;
--------------------------------------------------------------------------------
-- JTAG
-------------
-- FMC JTAG
	FMC_TCK	  <= C_TCK;
	FMC_TMS	  <= C_TMS;
	FMC_TDI	  <= C_TDI;
	C_TDO     <= FMC_TDO;
--PJTAG0
MIO26		   		<= JTAG_TCK; --tck
MIO27		   		<= JTAG_TDI; --tdi
JTAG_TDO      <= MIO28;  --tdo
MIO29		   		<= JTAG_TMS; --tms

JTAG_RTCK	  <= JTAG_TCK;
	-- JTAG_TRST			: in  STD_LOGIC;				
	-- JTAG_SRST			: in  STD_LOGIC;					

--------------------------------------------------------------------------------
--CAN
-------------
CAN_TX  <= SC18;
SC19    <= CAN_RX;
CAN_S	  <= SC16;
--------------------------------------------------------------------------------
--SFP
SFP1_TX_DIS		<= '0'; --, REV02:comment 
SFP2_TX_DIS		<= '0'; --, REV02:comment 
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--OSC
-------------
OSC_EN		  <= '1'; -- for the case we use it :)
--------------------------------------------------------------------------------
-- LEDs
-------------
-- GREEN D4
LED0      <=  blink_status(6) when intercpld_bpwr_i='0'       else    -- ******** -- power off
              blink_status(5) when iclld_rgpio_active='0'     else    -- *****ooo --inter cpld inactive
              blink_status(4) when por_p='1'                  else    -- ****oooo -- power reset
              blink_status(3) when usb_rst_p='1'              else    -- ***ooooo -- usb reset
              -- blink_status(2) when ???                        else    -- **oooooo --unused
              rgpio_out_data_i(4) when soc_rgpio_active='1'   else    -- user defined       
              blink_status(1) ;                                       -- *ooooooo
-- RED D5  
LED1      <=  blink_status(6) when intercpld_bpwr_i='0'       else    -- ******** -- power off
              blink_status(5) when iclld_rgpio_active='0'     else    -- *****ooo --inter cpld inactive
              blink_status(4) when por_p='1'                  else    -- ****oooo -- power reset
              blink_status(3) when pcie_rst_p='1'             else    -- ***ooooo -- pcie reset
              -- blink_status(2) when ???                        else    -- **oooooo --unused
              rgpio_out_data_i(5) when soc_rgpio_active='1'   else    -- user defined       
              blink_status(1) ;                                       -- *ooooooo
--SFP LEDS    
--SFP_LED0 is D1 Red   
--SFP_LED1 is D8 Green   
--SFP_LED2 is D9 Red   
--SFP_LED4 is D10 Green   
SFP_LED0	  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else rgpio_out_data_i(0) when soc_rgpio_active='1' else '0'; 
SFP_LED1	  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else rgpio_out_data_i(1) when soc_rgpio_active='1' else '0'; 
SFP_LED2	  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else rgpio_out_data_i(2) when soc_rgpio_active='1' else '0'; 
SFP_LED3	  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else rgpio_out_data_i(3) when soc_rgpio_active='1' else '0'; 
--PHY LEDS                                          
JLED1		    <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else (not PHY_LED0);
JLED2A		  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else PHY_LED1 ;
JLED2B		  <= '0'        when (por_resetn='0')else '0'   when ps_init_p='1'  else '0';
--XMOD2 (J28)
XMOD_E 		  <= slow_blink when (por_resetn='0')else blink when ps_init_p='1'  else not SC17 when XMOD_G='0' else SC17; --XMOD_G inverts XMOD LED so both combinations has same LED state




--------------------------------------------------------------------------------
--internal CLK
OSCInst0: OSCH
-- synthesis translate_off
GENERIC MAP ( 
	NOM_FREQ => "24.18" 
)
-- synthesis translate_on
PORT MAP (
	STDBY		=> '0', 
	OSC			=> osc_clk, 
	SEDSTDBY	=> open
);
clk 		<= osc_clk;
---------------------------------------
-- only for all led blinking sequences
led_conter: process(clk)
begin
	if (rising_edge(clk)) then
		led_cnt <= led_cnt + X"00000001";		
	end if;
end process;
blink 		  <= led_cnt(22);
slow_blink  <= led_cnt(25);

blink_status(0) <= '0';                                                             -- off
blink_status(1) <= led_cnt(25) and led_cnt(24) and led_cnt(23) and led_cnt(22);		  -- *ooooooo
blink_status(2) <= led_cnt(25) and led_cnt(24) and led_cnt(22);						          -- **oooooo
blink_status(3) <= led_cnt(25) and (led_cnt(24) or led_cnt(23)) and led_cnt(22);	  -- ***ooooo
blink_status(4) <= led_cnt(25) and led_cnt(22);										                  -- ****oooo
blink_status(5) <= (led_cnt(25) or (led_cnt(24) and led_cnt(23))) and led_cnt(22);	-- *****ooo
blink_status(6) <= led_cnt(22);                                                     -- ********
blink_status(7) <= '1';             


--------------------------------------------------------------------------------
--statistic and uart output on second XMOD: TODO replace with small processor system on next firmware update
uart_id_inst: uart_id
  generic map(
    DIVIDER  => 210,  --115200:210@24,18MHz CLK, --9600:2518@24,18MHz CLK, --4800:5038@24,18MHz CLK,--2400:10075@24,18MHz CLK,--1200:20150@24,18MHz CLK,
    TXT_STRING => CPLD_REV,
    STAT_SIZE => STAT_SIZE
  )
  port map( 
    CLK_IN              => clk,
    RESET_IN            => XMOD_G, -- one: RX-TX feedback, zero: start stream
    UART_TX_OUT         => XMOD_TXD,
    UART_RX_IN          => XMOD_RXD,
    STAT_POR_IN         => stat_trig_por_i,
    STAT_POR_OUT        => stat_por_i,
    STAT_PS_POR_IN      => stat_trig_ps_por_i,
    STAT_PS_POR_OUT     => stat_ps_por_i,
    STAT_PS_SRST_IN     => stat_trig_ps_srst_i,
    STAT_PS_SRST_OUT    => stat_ps_srst_i,
    STAT_PCIE_IN        => stat_trig_pcie_i,
    STAT_PCIE_OUT       => stat_pcie_i,
    STAT_USB_IN         => stat_trig_usb_i,
    STAT_USB_OUT        => stat_usb_i,
    STAT_PS_ERR_IN      => stat_trig_ps_err_i,
    STAT_PS_ERR_OUT     => stat_ps_err_i,
    STAT_PS_ERRSTAT_IN  => stat_trig_ps_errstat_i,
    STAT_PS_ERRSTAT_OUT => stat_ps_errstat_i,
    STAT_PS_INIT_IN     => stat_trig_ps_init_i,
    STAT_PS_INIT_OUT    => stat_ps_init_i,
    STAT_MPSTM_IN       => stat_mpstm_i
  );
  
--trigger one rising edge -> all converted  to pos
stat_trig_ps_srst_i       <= ps_srst_p    ; -- ps soft reset
stat_trig_ps_por_i        <= ps_por_p     ; -- ps power on reset
stat_trig_ps_err_i        <= ps_err_p     ; -- Err flag
stat_trig_ps_errstat_i    <= ps_err_stat_p; -- Err stat flag
stat_trig_ps_init_i       <= ps_init_p    ; -- Init flag
stat_trig_pcie_i          <= pcie_rst_p   ; -- pcie reset
stat_trig_usb_i           <= usb_rst_p    ; -- usb reset 
stat_trig_por_i           <= por_p        ; -- power on reset
stat_mpstm_i              <= intercpld_bpwr_i & iclld_rgpio_out(14 downto 12);                                            -- main power state machine, not correct state if 1.8V is disabled

--------------------------------------------------------------------------------
-- SoC/CPLD RGPIO 
-------------
-- <=rgpio_out_data_i(23 downto 12); --unused
-- <=rgpio_out_data_i(7 downto 6);   --unused
--  rgpio_out_data_i(5 downto 0)     -- LED control
gpio_switch : process(stat_usb_i,stat_pcie_i,stat_ps_errstat_i,stat_ps_err_i,stat_ps_init_i,stat_ps_por_i,stat_ps_srst_i,stat_por_i,rgpio_in_mux_i,iclld_rgpio_out,rgpio_out_data_i)
begin
  rgpio_in_data_i(23) <= JTAG_SRST; 
  rgpio_in_data_i(22) <= JTAG_TRST; 
  rgpio_in_data_i(21) <= FMC_CLKDIR; 
  rgpio_in_data_i(20) <= SD_WP; 
  rgpio_in_data_i(19) <= '0'; 
  rgpio_in_data_i(18) <= SW4; --!attention!S5-4 control FMC_VADJ
  rgpio_in_data_i(17) <= SW3; --S5-3
  rgpio_in_data_i(16) <= XMOD_G; 
  rgpio_in_data_i(15) <= PHY_LED2; 
  rgpio_in_data_i(14) <= PHY_LED1; 
  rgpio_in_data_i(13) <= PHY_LED0; 
  rgpio_in_data_i(12) <= CAN_FAULT; 
  rgpio_in_data_i(11 downto 8) <= rgpio_out_data_i(11 downto 8); --current mux
  rgpio_in_data_i(7 downto 0) <= iclld_rgpio_out(7 downto 0); -- dummy maybe change
  case rgpio_in_mux_i is
  when x"1" =>
    rgpio_in_data_i(7 downto 0) <= stat_por_i; 
  when x"2" =>
    rgpio_in_data_i(7 downto 0) <= stat_ps_srst_i; 
  when x"3" =>
    rgpio_in_data_i(7 downto 0) <= stat_ps_por_i; 
  when x"4" =>
    rgpio_in_data_i(7 downto 0) <= stat_ps_init_i; 
  when x"5" =>
    rgpio_in_data_i(7 downto 0) <= stat_ps_err_i; 
  when x"6" =>
    rgpio_in_data_i(7 downto 0) <= stat_ps_errstat_i; 
  when x"7" =>
    rgpio_in_data_i(7 downto 0) <= stat_pcie_i; 
  when x"8" =>
    rgpio_in_data_i(7 downto 0) <= stat_usb_i; 
  when others =>
  end case;
end process;
rgpio_in_mux_i <= rgpio_out_data_i(11 downto 8); 

rgpio_clk 		<= SC15;
SC13 			<= rgpio_txd;
rgpio_rxd 		<= SC14;
--------------------------------------------------------------------------------
SoC_MyRemoteGPIO_slave : RGPIO_top 
  Generic map(
    C_ADD_RESERVED  => 1,--0 disabled 1 enabled
    C_TYP           => 1  --0 Master, 1 Slave ,2 Both 
  )
  Port map( 
		-----------------------------------------------------
		--------------------FPGAIO-BUS Ports
		--
		-- Master 
		-- -external
		RGPIO_M_CLK   => open,--: out std_logic;   --  RGPIO Clock
		RGPIO_M_RX    =>'1',--: in  std_logic;   -- RGPIO RXD        
		RGPIO_M_TX    => open,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_M_OUT                   => open,             --: out std_logic_vector(23 downto 0); --user data
    RGPIO_M_IN                    => (others => '1'),  --: in std_logic_vector(23 downto 0); --user data
		-- -internal control         => ,
	  RGPIO_M_RESERVED_OUT          => open,  --: out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_RESERVED_IN           => (others => '1'),  --: in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_SLAVE_ACTIVATION_CODE => open,  --: out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
	  RGPIO_M_ENABLE                => '1',  --: in  std_logic; -- Enable RGPIO
	  RGPIO_M_USRCLK                => '1',  --: in  std_logic; -- RGPIO CLK
	  RGPIO_M_RESET_N               => '1',  --: in  std_logic; -- RGPIO Reset
		--  
		-- Slave 
		-- -external
		RGPIO_S_CLK   => rgpio_clk,--: in std_logic;  --  RGPIO Clock
		RGPIO_S_RX    => rgpio_rxd,--: in  std_logic;   -- RGPIO RXD        
		RGPIO_S_TX    => rgpio_txd,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_S_OUT                     => rgpio_out_data_i(23 downto 0), -- : out std_logic_vector(23 downto 0); --user data
    RGPIO_S_IN                      => rgpio_in_data_i(23 downto 0),  -- : in std_logic_vector(23 downto 0); --user data
		-- -internal control        
	  RGPIO_S_RESERVED_OUT            => open, -- : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_RESERVED_IN             => "1111", --  : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_MASTER_ACTIVATION_CODE  => open, --- : out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
	  RGPIO_S_ENABLED                 => soc_rgpio_active -- : out  std_logic -- Enable RGPIO
  
	);

--------------------------------------------------------------------------------
--Inter CPLD RGPIO runs only if 1.8V is enabled (bank io power)
-------------
iclld_rgpio_clk 	<= X1;	
X2 <= iclld_rgpio_txd ;		
iclld_rgpio_rxd 	<= X3;
	
iclld_rgpio_in(31 downto 2)  <= x"FF00FF0" & "00"; 	--unused
-- iclld_rgpio_in(1 downto 0) <= --used for FMC Sanity check
---------------
InterCPLD_MyRemoteGPIO_slave : RGPIO_top 
  Generic map(
    C_ADD_RESERVED  => 1,--0 disabled 1 enabled
    C_TYP           => 1  --0 Master, 1 Slave ,2 Both 
  )
  Port map( 
		-----------------------------------------------------
		--------------------FPGAIO-BUS Ports
		--
		-- Master 
		-- -external
		RGPIO_M_CLK   => open,--: out std_logic;   --  RGPIO Clock
		RGPIO_M_RX    =>'1',--: in  std_logic;   -- RGPIO RXD        
		RGPIO_M_TX    => open,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_M_OUT                   => open,             --: out std_logic_vector(23 downto 0); --user data
    RGPIO_M_IN                    => (others => '1'),  --: in std_logic_vector(23 downto 0); --user data
		-- -internal control         => ,
	  RGPIO_M_RESERVED_OUT          => open,  --: out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_RESERVED_IN           => (others => '1'),  --: in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_M_SLAVE_ACTIVATION_CODE => open,  --: out  std_logic_vector(3 downto 0); -- activation code from external slavefor information only
	  RGPIO_M_ENABLE                => '1',  --: in  std_logic; -- Enable RGPIO
	  RGPIO_M_USRCLK                => '1',  --: in  std_logic; -- RGPIO CLK
	  RGPIO_M_RESET_N               => '1',  --: in  std_logic; -- RGPIO Reset
		--  
		-- Slave 
		-- -external
		RGPIO_S_CLK   => iclld_rgpio_clk,--: in std_logic;  --  RGPIO Clock
		RGPIO_S_RX    => iclld_rgpio_rxd,--: in  std_logic;   -- RGPIO RXD        
		RGPIO_S_TX    => iclld_rgpio_txd,--: out std_logic;   -- RGPIO TXD 
    
		-- -internal data
    RGPIO_S_OUT                     => iclld_rgpio_out(23 downto 0), -- : out std_logic_vector(23 downto 0); --user data
    RGPIO_S_IN                      => iclld_rgpio_in(23 downto 0),  -- : in std_logic_vector(23 downto 0); --user data
		-- -internal control        
	  RGPIO_S_RESERVED_OUT            => iclld_rgpio_out(27 downto 24), -- : out  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_RESERVED_IN             => iclld_rgpio_in(27 downto 24), --  : in  std_logic_vector(3 downto 0); -- reserved for future usage
	  RGPIO_S_MASTER_ACTIVATION_CODE  => iclld_rgpio_out(31 downto 28), --- : out  std_logic_vector(3 downto 0);  -- activation code from external master for information only
	  RGPIO_S_ENABLED                 => iclld_rgpio_active -- : out  std_logic -- Enable RGPIO
  
	);

--------------------------------------------------------------------------------
END ARCHITECTURE translated;