----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 01.04.2019
-- Design Name: 
-- Module Name: 
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
-- use ieee.STD_LOGIC_arith.all;
use ieee.STD_LOGIC_unsigned.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity uart_id is
  generic (
    DIVIDER : integer := 2518;  --115200:210@24,18MHz CLK, --9600:2518@24,18MHz CLK, --4800:5038@24,18MHz CLK,--2400:10075@24,18MHz CLK,--1200:20150@24,18MHz CLK,
    TXT_STRING : string := "TEBF0808-x_CPLD-REV??DEV";
    STAT_SIZE : integer :=6
  );
  port ( 
    CLK_IN : in std_logic;
    RESET_IN : in std_logic; -- one: RX-TX feedback, zero: start stream
    UART_TX_OUT : out std_logic;
    UART_RX_IN : in std_logic;
    --
    STAT_POR_IN    : in std_logic;
    STAT_POR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_POR_IN    : in std_logic;
    STAT_PS_POR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_SRST_IN    : in std_logic;
    STAT_PS_SRST_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PCIE_IN    : in std_logic;
    STAT_PCIE_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_USB_IN    : in std_logic;
    STAT_USB_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_ERR_IN    : in std_logic;
    STAT_PS_ERR_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_ERRSTAT_IN    : in std_logic;
    STAT_PS_ERRSTAT_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_PS_INIT_IN    : in std_logic;
    STAT_PS_INIT_OUT   : out std_logic_vector(STAT_SIZE+2-1 downto 0);
    STAT_MPSTM_IN   : in std_logic_vector(4-1 downto 0)
  );
end uart_id;

architecture Behavioral of uart_id is
function to_slv(str : string) return std_logic_vector is
  alias str_norm : string(str'length downto 1) is str;
  variable tmp : std_logic_vector(8-1 downto 0);
  variable tmp2 : std_logic_vector(8-1 downto 0);
  variable res_v : std_logic_vector(8 * str'length - 1 downto 0);
  variable send : std_logic_vector(8 * str'length+ str'length*2 - 1 downto 0);
begin
  for idx in str_norm'range loop
    -- res_v(8 * idx - 1 downto 8 * idx - 8) := 
      -- std_logic_vector(to_unsigned(character'pos(str_norm(idx)), 8));
    tmp := std_logic_vector(to_unsigned(character'pos(str_norm(idx)), 8));
    for i2 in 0 to 8-1 loop
      tmp2(i2) := tmp(7-i2);
    end loop;
    res_v(8 * idx - 1 downto 8 * idx - 8) := tmp2;
  end loop;
  
  for idx in 0 to str'length-1 loop
    send((idx+1)*(10)-1 downto idx*(10)) := '0'& res_v((idx+1)*8-1 downto idx*8) & '1';
  end loop;
  
  return send;
end function;

function cvString (N, L: integer) return String
-- Accepts a number N to convert into a string of length L.  If the resulting
-- string is too small, you get the least-significant digits of N in the
-- returned string.
is
  variable Temp: integer := N;
  variable Result: String(1 to L) := (others => ' ');
begin
  for i in Result'reverse_range loop
    Result(i) := Character'Val((Temp mod 10) + Character'Pos('0'));
    Temp := Temp/10;
    exit when Temp = 0;
  end loop;
  return Result;
end cvString;


---------------------------------------------------------

component statistic is
generic (
  SIZE : integer := 4
);
port ( 
      CLK_IN : in std_logic;
      RESET_IN : in std_logic;
      TRIGGER_IN : in std_logic;
      COUNT_OUT : out std_logic_vector(SIZE-1 downto 0);
      OVERFLOW_OUT : out std_logic;
      DETECTED_OUT : out std_logic);
end component;
---------------------------------------------------------
signal stat_reset_i : std_logic;
signal stat_por_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_srst_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_por_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_pcie_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_usb_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_err_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_errstat_i : std_logic_vector(STAT_SIZE+2-1 downto 0);
signal stat_ps_init_i : std_logic_vector(STAT_SIZE+2-1 downto 0);

signal stat_por_v : std_logic_vector(1 downto 0);
signal stat_ps_srst_v : std_logic_vector(1 downto 0);
signal stat_ps_por_v : std_logic_vector(1 downto 0);
signal stat_pci_v : std_logic_vector(1 downto 0);
signal stat_usb_v : std_logic_vector(1 downto 0);
signal stat_ps_err_v : std_logic_vector(1 downto 0);
signal stat_ps_errstat_v : std_logic_vector(1 downto 0);
signal stat_ps_init_v : std_logic_vector(1 downto 0);
signal stat_mpstm_v : std_logic_vector(1 downto 0);
-----------------------------------------------------
constant mp : string := "|MP:";
constant stat : string := "|St:";
constant stat_1 : string := "-";
constant stat_pwr : string := "POR";
constant stat_usb : string := "USB";
constant stat_pci : string := "PCI";
constant stat_psr : string := "PSR";
constant stat_phr : string := "PHR";
constant stat_psi : string := "PIN";
constant stat_pse : string := "PEE";
constant stat_pss : string := "PES";
constant stat_2 : string := ";";


constant id_start : std_logic_vector(4*10-1 downto 0) := "1"& x"FF" & "1" & "1"& x"FF" & "1" & "1"& x"FF" &"1" & "1"& x"FF" & "1" ;
constant id_newline : std_logic_vector(1*10-1 downto 0) := "0"& x"B0" & "1";
-- constant id_newline : std_logic_vector(2*10-1 downto 0) := "0" & x"50" & "1" & "0"& x"B0" & "1";
constant id_end : std_logic_vector(1*10-1 downto 0) := "1"& x"FF" & "1";
signal uart_output : std_logic;
-- signal firmware_id : std_logic_vector(1*id_start'length + 2*id_newline'length + 1*id_end'length 
signal firmware_id : std_logic_vector(1*id_start'length + 1*id_newline'length + 1*id_end'length 
                                      + (10*TXT_STRING'length) 
                                      + (10*mp'length) + (1*10) + (10*stat_1'length) + (1*10)
                                      + (10*stat'length) 
                                      + (10*stat_psi'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_pse'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_pss'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_psr'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_phr'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_pwr'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_usb'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      + (10*stat_pci'length) + (1*10)+ (10*stat_1'length) + (2*10) + (10*stat_2'length)
                                      -1 downto 0);

signal fid_count : std_logic_vector (15 downto 0) := (others => '0');
signal next_fid : std_logic := '0';
signal c_mux_counter : std_logic_vector((2**10)-1 downto 0) := (others => '1');--:= std_logic_vector(to_unsigned(firmware_id'length-1, (2**10)));
signal n_mux_counter : std_logic_vector((2**10)-1 downto 0);
-----------------------------------------------------
begin

stat_por_v <= '0' & STAT_POR_IN;
stat_ps_srst_v <= '0' & STAT_PS_SRST_IN;
stat_ps_por_v <= '0' & STAT_PS_POR_IN;
stat_pci_v <= '0' & STAT_PCIE_IN;
stat_usb_v <= '0' & STAT_USB_IN;
stat_ps_err_v <= '0' & STAT_PS_ERR_IN;
stat_ps_errstat_v <= '0' & STAT_PS_ERRSTAT_IN;
stat_ps_init_v <= '0' & STAT_PS_INIT_IN;
stat_mpstm_v <= '0' & STAT_MPSTM_IN(3);


firmware_id <= id_start 
& id_newline
& to_slv(TXT_STRING) 
& to_slv(mp) & to_slv(cvString(to_integer(unsigned(stat_mpstm_v)),1)) &  to_slv(stat_1) & to_slv(cvString(to_integer(unsigned(STAT_MPSTM_IN(2 downto 0))),1)) 
& to_slv(stat) 
& to_slv(stat_psi) & to_slv(cvString(to_integer(unsigned(stat_ps_init_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_ps_init_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_pse) & to_slv(cvString(to_integer(unsigned(stat_ps_err_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_ps_err_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_pss) & to_slv(cvString(to_integer(unsigned(stat_ps_errstat_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_ps_errstat_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_psr) & to_slv(cvString(to_integer(unsigned(stat_ps_srst_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_ps_srst_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_phr) & to_slv(cvString(to_integer(unsigned(stat_ps_por_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_ps_por_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_pwr) & to_slv(cvString(to_integer(unsigned(stat_por_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_por_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_usb) & to_slv(cvString(to_integer(unsigned(stat_usb_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_usb_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& to_slv(stat_pci) & to_slv(cvString(to_integer(unsigned(stat_pci_v)),1)) &  to_slv(stat_1)  & to_slv(cvString(to_integer(unsigned(stat_pcie_i(STAT_SIZE-1 downto 0))),2)) & to_slv(stat_2)
& id_end;

firmwareid_reg :  process (CLK_IN)
begin
   if (rising_edge(CLK_IN)) then
        -- if RESET_IN='1' then
          -- c_mux_counter <= std_logic_vector(to_unsigned(firmware_id'length-1, c_mux_counter'length));
        -- else
          c_mux_counter <= n_mux_counter;  
        -- end if;    
   end if;
end process;
n_mux_counter <= c_mux_counter - std_logic_vector(to_unsigned(1, fid_count'length)) when next_fid='1' else c_mux_counter;
-- firmwareid_mux :  process (c_mux_counter)
-- begin
  -- for index in firmware_id'length-1 downto 0 loop
    -- if (index=to_integer(unsigned(c_mux_counter))) then   
      -- uart_output <=firmware_id(index);
      -- exit;
    -- else
      -- uart_output <= '1';
    -- end if;
  -- end loop;
-- end process;
uart_output <=firmware_id(to_integer(unsigned(c_mux_counter)));


firmwareid_enable :  process (CLK_IN)
begin
   if (rising_edge(CLK_IN)) then
    if (fid_count=std_logic_vector(to_unsigned(DIVIDER, fid_count'length))) then
      fid_count <=x"0000";
      next_fid <= '1';
    else
      next_fid <= '0';
      fid_count <= fid_count + std_logic_vector(to_unsigned(1, fid_count'length));
    end if;
   end if;
end process;
-- UART_TX_OUT <=  firmware_id(firmware_id'length-1) when RESET_IN='0' else UART_RX_IN;
UART_TX_OUT <=  uart_output when RESET_IN='0' else UART_RX_IN;

--------------------------------------------------------------------------------
-- statistic only:
stat_reset_i <= '0';


statistic_pwr_rst: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_POR_IN,
      COUNT_OUT  => stat_por_i(stat_por_i'length-3 downto 0),
      OVERFLOW_OUT => stat_por_i(stat_por_i'length-2),
      DETECTED_OUT => stat_por_i(stat_por_i'length-1)
);

statistic_ps_srst: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PS_SRST_IN,
      COUNT_OUT  => stat_ps_srst_i(stat_ps_srst_i'length-3 downto 0),
      OVERFLOW_OUT => stat_ps_srst_i(stat_ps_srst_i'length-2),
      DETECTED_OUT => stat_ps_srst_i(stat_ps_srst_i'length-1)
);

statistic_ps_por: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PS_POR_IN,
      COUNT_OUT  => stat_ps_por_i(stat_ps_por_i'length-3 downto 0),
      OVERFLOW_OUT => stat_ps_por_i(stat_ps_por_i'length-2),
      DETECTED_OUT => stat_ps_por_i(stat_ps_por_i'length-1)
);

statistic_ps_err: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PS_ERR_IN,
      COUNT_OUT  => stat_ps_err_i(stat_ps_err_i'length-3 downto 0),
      OVERFLOW_OUT => stat_ps_err_i(stat_ps_err_i'length-2),
      DETECTED_OUT => stat_ps_err_i(stat_ps_err_i'length-1)
);

statistic_ps_errstat: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PS_ERRSTAT_IN,
      COUNT_OUT  => stat_ps_errstat_i(stat_ps_errstat_i'length-3 downto 0),
      OVERFLOW_OUT => stat_ps_errstat_i(stat_ps_errstat_i'length-2),
      DETECTED_OUT => stat_ps_errstat_i(stat_ps_errstat_i'length-1)
);

statistic_ps_init: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PS_INIT_IN,
      COUNT_OUT  => stat_ps_init_i(stat_ps_init_i'length-3 downto 0),
      OVERFLOW_OUT => stat_ps_init_i(stat_ps_init_i'length-2),
      DETECTED_OUT => stat_ps_init_i(stat_ps_init_i'length-1)
);

statistic_pcie: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_PCIE_IN,
      COUNT_OUT  => stat_pcie_i(stat_pcie_i'length-3 downto 0),
      OVERFLOW_OUT => stat_pcie_i(stat_pcie_i'length-2),
      DETECTED_OUT => stat_pcie_i(stat_pcie_i'length-1)
);
statistic_usb: statistic
generic map(
  SIZE => STAT_SIZE
)
port map( 
      CLK_IN  => CLK_IN,
      RESET_IN  => stat_reset_i,
      TRIGGER_IN => STAT_USB_IN,
      COUNT_OUT  => stat_usb_i(stat_usb_i'length-3 downto 0),
      OVERFLOW_OUT => stat_usb_i(stat_usb_i'length-2),
      DETECTED_OUT => stat_usb_i(stat_usb_i'length-1)
);
STAT_POR_OUT <= stat_por_i;
STAT_PS_POR_OUT <= stat_ps_por_i;
STAT_PS_SRST_OUT <= stat_ps_srst_i;
STAT_PCIE_OUT <= stat_pcie_i;
STAT_USB_OUT <= stat_usb_i;
STAT_PS_ERR_OUT <= stat_ps_err_i;
STAT_PS_ERRSTAT_OUT <= stat_ps_errstat_i;
STAT_PS_INIT_OUT <= stat_ps_init_i;


end Behavioral;
